#include <ncbi.h>
#include "blastapp.h"
#include "aabet.h"

/* seqfilter -- filter a sequence */
int
seqfilter(cmd, outtab, intab, seqname, seqnamelen, inseq, inseqlen, outseq, outseqlen)
	CharPtr	cmd;
	CharPtr	outtab; /* translation table for output */
	int		PNTR intab; /* translation table for input */
	CharPtr		seqname;
	size_t		seqnamelen;
	UcharPtr	inseq, outseq;
	size_t	inseqlen, PNTR outseqlen;
{
	FILE	*fp;
	CharPtr	filterdir;
	static char	permenv[2048], permenv2[2048];
	char	cmdbuf[4*FILENAME_MAX+1024];
	char	inname[FILENAME_MAX+1];
	int		omask;
	unsigned char	xltab[1<<CHAR_BIT];
	register UcharPtr	cp, cpmax;
	CharPtr	saveenv = NULL, tcp;
	CharPtr	name = NULL;
	UcharPtr	seq = NULL;
	size_t		namelen = 0, namemax = 0, seqlen = 0, seqmax = 0;
	int		i;

	if (cmd == NULL || cmd[0] == NULLB) {
		if (inseq != outseq)
			memcpy((char *)outseq, (char *)inseq, inseqlen);
		*outseqlen = inseqlen;
		return 0;
	}

	/* No absolute or relative pathnames are permitted */
	errno = 0;
	if (cmd[0] == DIRDELIMCHR || cmd[0] == '.')
		return 1;

	omask = umask(077);

	/* Restrict system() calls to a particular directory */
	saveenv = getenv("PATH");
	filterdir = getenv("BLASTFILTER");
	if (filterdir == NULL)
		filterdir = BLASTFILTER;
	if (saveenv != NULL) {
		tcp = ckalloc(i = strlen(saveenv) + 1);
		memcpy(tcp, saveenv, i);
		saveenv = tcp;
	}
	sprintf(permenv, "PATH=%s", filterdir);
	putenv(permenv);

	inname[0] = NULLB;
	if (tmpnam(inname) == NULL)
		goto ErrReturn;
	fp = fopen(inname, "w");
	if (fp == NULL)
		goto ErrReturn;
	if (putfasta(outtab, NULL, 0, seqname, seqnamelen, inseq, inseqlen, fp) != 0)
		goto ErrReturn;
	fclose(fp);

	sprintf(cmdbuf, cmd, inname, inname, inname, inname, inname, inname);

	fp = popen(cmdbuf, "r");
	if (fp == NULL)
		goto ErrReturn;
	if (saveenv != NULL) {
		sprintf(permenv2, "PATH=%s", saveenv);
		putenv(permenv2);
		free(saveenv);
	}

	for (i = 0; i < DIM(xltab); ++i)
		if (!isspace(i) && i != '\n' && i != '\r')
			xltab[i] = i;
	for (i = 'a'; i <= 'z'; ++i)
		xltab[i] = toupper(i);

	if (getfasta(xltab, &name, &namemax, &namelen, &seq, &seqmax, &seqlen, fp) != 0)
		goto ErrReturn;
	fflush(stderr);
	if (echofilter_flag)
		putfasta(xltab, NULL, 0, name, namelen, seq, seqlen, stdout);
	if (pclose(fp) != 0)
		goto ErrReturn;
	if (seqlen > inseqlen)
		goto ErrReturn;
	free(name);
	cp = seq, cpmax = seq + seqlen;
	while (cp < cpmax)
		*outseq++ = intab[*cp++];
	free(seq);
	*outseqlen = seqlen;
	unlink(inname);
	(void) umask(omask);

	return 0;

ErrReturn:
	fflush(stdout);
	(void) umask(omask);
	if (errno != 0)
		perror("seqfilter");
	fflush(stderr);
	if (inname[0])
		unlink(inname);
	return 1;
}


typedef struct {
	CharPtr	name;
	int		residue_type;
	CharPtr	command;
	} FilterList, PNTR FilterListPtr;
		
static FilterList	filterlist[] = {
		{ "seg", AMINO_ACID_SEQTYPE, "seg %s -x" } ,

		{ "xnu", AMINO_ACID_SEQTYPE, "xnu %s" } ,

		{ "seg+xnu", AMINO_ACID_SEQTYPE, "seg %s -x | xnu -" } ,

		{ "xnu+seg", AMINO_ACID_SEQTYPE, "xnu %s | seg - -x" } ,

		{ "none", AMINO_ACID_SEQTYPE, "" } ,
		{ "none", NUCLEIC_ACID_SEQTYPE, "" }
	};

CharPtr _cdecl
pick_filter(str, restype)
	register CharPtr	str;
	register int	restype;
{
	register int		i, j;
	CharPtr	cp;
	char	estr[4096];

	for (i = 0; i < DIM(filterlist); ++i)
		if (restype == filterlist[i].residue_type
				&& str_casecmp(str, filterlist[i].name) == 0)
			return filterlist[i].command;

	cp = strchr(str, '%');
	if (cp != NULL && cp[1] == 's')
		return str;

	strcpy(estr, "Invalid filter option:  ``%s''.  If spelled correctly, the specified option simply may not be applicable to the type of query sequence used by this program.  The valid filters known by this program are:  ");

	for (i = j = 0; i < DIM(filterlist); ++i)
		if (restype == filterlist[i].residue_type) {
			if (j++ > 0)
				strcat(estr, ", ");
			strcat(estr, filterlist[i].name);
		}

	bfatal(ERR_INVAL, estr, str);
}
